@@ -1,5 +1,5 @@
 #
-# $Date: 2009-01-02 20:14:31 $
+# $Date: 2009-01-11 21:41:07 $
 #
 # Copyright (c) 2007-2008 Alexandre Aufrere
 # Licensed under the terms of the GPL (see perldoc MRIM.pm)
@@ -146,7 +146,7 @@ sub set_status {
 
 package Net::MRIM;
 
-our $VERSION='1.10';
+our $VERSION='1.11';
 
 =pod
 
@@ -271,6 +271,7 @@ use constant {
   CONTACT_FLAG_SMS	=> 0x00100000,
  MRIM_CS_ADD_CONTACT_ACK	=> 0x101A,
   CONTACT_OPER_SUCCESS	=> 0x00000000,
+  CONTACT_OPER_USER_EXISTS	=> 0x00000005,
  MRIM_CS_AUTHORIZE		=> 0x1020,	# C -> S, LPS user
  MRIM_CS_MODIFY_CONTACT		=> 0x101B,	# C -> S, UL id, UL flags, UL group_id, LPS email, LPS name, LPS unused
  MRIM_CS_MODIFY_CONTACT_ACK	=> 0x101C,
@@ -411,9 +412,12 @@ sub send_message {
 }
 
 # send SMS
+# the implementation is awful: it adds and then remove an SMS entry to the contact list, and tries to send SMS in between.
+# why ? i'm French, live in France, and don't have any access to a russian mobile phone... so it's impossible for me to test, i just use some web literature i found on the topic
+# wish to help ? contact me - aau@cpan.org (vozmozhno i po-russkiy - lyudi! pomogite! ;-))))
 sub send_sms {
 	my ($self,$numberto,$message)=@_;
-	my $data=new Net::MRIM::Message();
+	my $dontremove=0;
 	print "DEBUG [send SMS]: $message\n" if ($self->{_debug});
 	# first, we should "add" it as SMS contact...
 	my $data=pack("V",CONTACT_FLAG_SMS).pack("V",0xffffffff)._to_lps($numberto)._to_lps("SMS")._to_lps($numberto);
@@ -422,19 +426,33 @@ sub send_sms {
 	my ($msgrcv,$datarcv,$dlen)=_receive_data($self);
 	my @datas=_from_mrim_us("uu",$datarcv);
 	my $cid=$datas[1];
-	if ($datas[0] != CONTACT_OPER_SUCCESS) {
+	# This is ugly: in case some message is in between, return without sending the SMS actually
+	if ($msgrcv != MRIM_CS_ADD_CONTACT_ACK) {
+		print "DEBUG [send SMS]: $message was NOT sent\n" if ($self->{_debug});
+		return _analyze_received_data($self,$msgrcv,$datarcv,$dlen);
+	}
+	# In case adding the contact failed, return without sending the SMS actually, but with an error message
+	if (($datas[0] != CONTACT_OPER_SUCCESS)&&($datas[0] != CONTACT_OPER_USER_EXISTS)) {
+		my $data=new Net::MRIM::Message();
 		$data->set_server_msg($data->{TYPE_SERVER_NOTIFY},$self->{_login},"CONTACT_OPER_ERROR_SMS: Error adding contact for SMS sending");
 		return $data;
 	}
+	$dontremove=1 if ($datas[0] == CONTACT_OPER_USER_EXISTS);
 	$data=pack("V",0)._to_lps($numberto)._to_lps($message);
 	$self->{_sock}->send(_make_mrim_packet($self,MRIM_CS_SMS,$data));
 	$self->{_seq_real}++;
-	($msgrcv,$datarcv,$dlen)=_receive_data($self);
+	my ($msgrcvsms,$datarcvsms,$dlensms)=_receive_data($self);
+	# if contact was already in contact list, do not try to remove it.
+	return _analyze_received_data($self,$msgrcvsms,$datarcvsms,$dlensms) if ($dontremove==1);
 	$data=pack("V",$cid).pack("V",CONTACT_FLAG_SMS|CONTACT_FLAG_REMOVED).pack("V",0xffffffff)._to_lps($numberto)._to_lps("SMS")._to_lps($numberto);
 	$self->{_sock}->send(_make_mrim_packet($self,MRIM_CS_MODIFY_CONTACT,$data));
 	$self->{_seq_real}++;
 	($msgrcv,$datarcv,$dlen)=_receive_data($self);
-	return _analyze_received_data($self,$msgrcv,$datarcv,$dlen);
+	# This is ugly: in case some message is in between, return without knowing if the SMS was actually sent
+	if ($msgrcv != MRIM_CS_MODIFY_CONTACT_ACK) {
+		return _analyze_received_data($self,$msgrcv,$datarcv,$dlen);
+	}
+	return _analyze_received_data($self,$msgrcvsms,$datarcvsms,$dlensms);
 }
 
 # to authorize a user to add us to the contact list
@@ -694,6 +712,7 @@ sub _analyze_received_data {
 			$name=~s/\n//g;
 			print "DEBUG: Found contact $name of id $email flags $flags $sflags $status $group unknown: $unk clen $clen dlen $dlen\n" if ($self->{_debug});
 			$name=$email if (length($name)<1);
+			$status=STATUS_OFFLINE if (($flags==CONTACT_FLAG_REMOVED)||($flags==CONTACT_FLAG_SMS)||($flags==(CONTACT_FLAG_SMS|CONTACT_FLAG_REMOVED))); # to take care about SMS contacts, if any
 			$contacts->{$email}=new Net::MRIM::Contact($email,$name,$status) if (($status != STATUS_OFFLINE)&&($status != STATUS_UNDETERMINED)&&(length($email)>1));
 			$all_contacts->{$email}=$i;
 			$clen=16+length($name)+length($email)+length($unk)+$clen;
@@ -785,6 +804,8 @@ sub _analyze_received_data {
 		my @datas=_from_mrim_us("u",$datarcv);
 		# actually, MRIM seems to return always "1"... so i leave the outpout only for debug
 		$data->set_message("DEBUG",$self->{_login},"SMS ACK: $datas[0]") if ($self->{_debug});
+		# wild guess that "0" should mean "SUCCESS"
+		$data->set_server_msg($data->{TYPE_SERVER_NOTIFY},$self->{_login},"SMS_SENT") if ($datas[0]==0);
 	} else {
 		$data->set_message("DEBUG",$self->{_login},$datarcv) if ($self->{_debug});
 	}
